*! author: Hung-Jen Wang, hjwang@econ.sinica.edu.tw
*! public2: update wholesale and land prices
*! update: 25Feb04, on line 192-193.
*! update: 27Feb04, on line 756.

** This program calculates Tobin's Q of Taiwanese
** manufacturing firms.  Please see the accompanied Readme file.

clear
set memory 3m
set more off
capture log close


/******************************************************/
/******************************************************/


/*********** setup program parameters *****************/

    /* -- to use this program: need to download all data starting from 1981 and up to the year of interest -- */

    /* -- you may want to customize the following three lines -- */

insheet using sample2.txt, double  /* read in the dataset */
local begyr 1988 /* the beginning year of computing q */
local endyr 1996 /* the ending year of computing q */


    /* --- don't change the following lines unless you know what you are doing --- */

local upland 2       /* upland and dwnland define the "trade land excessively" firms */
local dwnland -0.5
local oldlive 35 /* the remaining life of the pre-1979 asset counted from 1979 */
                 /* "6" is based on the observation from firms not reporting */
                 /* revaluation that their asset life is about 10 years.  Then */
                 /* take 2/3 of the 10. */
local mixrate 0.35 /* the portion of old asset in all asset, used only in odd cases */


scalar oldlpB = 1968 /* land price between oldlpB and 1979 is used to get an average.  min(oldlpB) = 1968 */

drop if month != 12  /* just for cautious */


/**** correct TEJ's error ****/

quie replace revland = 43402 if cnum == 1435 & month == 12 & (yr == 1988 | yr == 1985)
quie replace revother = 5766 if cnum == 1435 & month == 12 & yr == 1988
quie replace revother = 8100 if cnum == 1435 & month == 12 & yr == 1985


/**************  this section creates whole sale price level **/

scalar p1973 =  52.22
scalar p1974 =  73.41
scalar p1975 =  69.69
scalar p1976 =  71.62
scalar p1977 =  73.60
scalar p1978 =  76.20
scalar p1979 =  86.73
scalar p1980 = 105.42
scalar p1981 = 113.46
scalar p1982 = 113.25
scalar p1983 = 111.92
scalar p1984 = 112.44
scalar p1985 = 109.53
scalar p1986 = 105.87
scalar p1987 = 102.42
scalar p1988 = 100.82
scalar p1989 = 100.45
scalar p1990 =  99.84
scalar p1991 = 100.00
scalar p1992 =  96.33
scalar p1993 =  98.75
scalar p1994 = 100.89
scalar p1995 = 108.33
scalar p1996 = 107.24
scalar p1997 = 106.76
scalar p1998 = 107.40
scalar p1999 = 102.51
scalar p2000 = 104.37
scalar p2001 = 102.97

quie gen price = .
global endyrg = `endyr'

capture program drop makep
program define makep
 local i  1973
 while `i' <= $endyrg {
 quie replace price = p`i' if yr == `i'
 local i = `i' + 1
 }
end

makep

/**************/

/**************  this section creates land price level. **/

scalar L1968 = 13.50
scalar L1969 = 13.64
scalar L1970 = 11.63
scalar L1971 = 10.71
scalar L1972 =  9.81
scalar L1973 = 10.68
scalar L1974 = 10.79
scalar L1975 = 29.63
scalar L1976 = 33.63
scalar L1977 = 32.63
scalar L1978 = 29.20
scalar L1979 = 30.27
scalar L1980 = 30.29
scalar L1981 = 31.09
scalar L1982 = 28.83
scalar L1983 = 28.06
scalar L1984 = 26.35
scalar L1985 = 26.22
scalar L1986 = 25.16
scalar L1987 = 70.08
scalar L1988 = 65.58
scalar L1989 = 85.04
scalar L1990 = 82.78
/* the old price imputed from government data */
* scalar L1991 = 100.00
* scalar L1992 = 94.03
* scalar L1993 = 90.60
* scalar L1994 = 108.93
* scalar L1995 = 100.29
* scalar L1996 = 99.28
* scalar L1997 = 104.42
* scalar L1998 = 105.53
* scalar L1999 = 104.40
* scalar L2000 = 105.68
* scalar L2001 = 105.96
/* the following is the new data from Xin-Yi Real Estate */
/* see housep_processed.dta */
scalar L1991 = 100.00
scalar L1992 = 108.10
scalar L1993 = 110.41
scalar L1994 = 110.79
scalar L1995 = 104.65
scalar L1996 =  98.37
scalar L1997 =  99.11
scalar L1998 =  97.44
scalar L1999 =  94.50
scalar L2000 =  91.92
scalar L2001 =  84.44


quie gen landP = .

capture program drop makeL
program define makeL
 local i  1968
 while `i' <= $endyrg {
 quie replace landP = L`i' if yr == `i'
 local i = `i' + 1
 }
end

makeL

capture program drop makeOP /* to get the average of the old land price before 1979 */
program define makeOP
 local dumyP 0
 local i =oldlpB
 while `i' <= 1979 {
 local dumyP = `dumyP' + L`i'
 local i = `i' + 1
 }
 scalar oldLP = `dumyP'/(1979-oldlpB+1) /* old Land Price */
end

makeOP


/******* Begin Data Filtering *******/


drop if (cnum <= 2599) & (cnum >= 2500) /* excluding construction industry */
drop if yr > `endyr'
 /* drop obs if important vars are missing */
drop if (fa == .) | (accdepre == .) | (land == .)
drop if (fa == 0) | (buyfa == 0)  /* an indication of data error; ex 2347 in 1988 */
drop if (accdepre == 0) /* impossible, must be data error.  ex 1209 in 1981 */
drop if cnum == 9901  /* very odd case */
drop if refa > 0 & refa !=. & revland == 0 & revplant == 0 & revmach == 0 & revother == 0 /* cnum = 1204*/
drop if fa < refa /* impossible; cnum == 1438, 1710 */
* drop if revland/land > 600 & land < 2000 /* unreasonable, ex 1903 */
drop if (land~=0 & revland/land > 600) & land < 2000 /* unreasonable, ex 1903; updated on 25Feb04. */



   /* if at least one of them is not missing, set the missings to 0. */
quie gen tem1 = 1 if revland == . & revplant == . & revmach == . & revother == .
quie recode tem1 .=0 /* tem1 = 1 if all are missing; = 0 if at least one of them is not missing */
quie recode revland .=0 if tem1 == 0 /* if not all of them are ., set it to 0 */
quie recode revplant .=0 if tem1 == 0
quie recode revmach .=0 if tem1 == 0
quie recode revother .=0 if tem1 == 0

  /* if refa is missing but the sum of the components are not, set refa to the sum */
quie replace refa = (revland + revplant + revmach + revother) if refa == .
quie gen tem2 = 1 if refa == .
quie egen tem3 = sum(tem2), by(cnum) /* tem3 == . if all refa == ., and tem3 != . if some refa != . */
quie replace refa = 0 if refa == . & tem3 == . /* if all refa == ., replace missing of refa by 0 */
drop if refa == .
drop tem1 tem2 tem3


 /* if refa is > 0 and not missing, but all the components are missing, use simple interpolation */
gsort cnum -yr
quie by cnum: gen tem1 = 1 if refa !=0 & refa !=. & revland ==.
quie by cnum: replace revland = (revland[_n-1] + revland[_n-2] )/2 if tem1 == 1
drop tem1
sort cnum yr


 /* replace missing values by 0 for selected variables. */
quie mvencode salefa lstdisfa revland revplant revmach revother, mv(0) override

sort cnum yr month

 /* select firms that have complete obs in the desired range */
quie gen count1 = 1 if yr >=`begyr'
recode count1 .=0
quie egen sum1 = sum(count1), by(cnum)

drop if sum1 != `endyr' - `begyr' + 1  /* missing obs in the desired range */
drop count1 sum1

 /* double check for discontinuous odd cases */
sort cnum yr
quie by cnum: gen kill1 = 1 if (yr >= `begyr') & (yr[_n+1] != yr[_n]+1)
egen count1 = count(kill1), by(cnum)
drop if count1 !=1

drop kill1 count1
sort cnum yr month

 /* delete unbalanced portion of the earlier periods */
quie by cnum: gen kill1 = yr if (yr < `begyr') & (yr[_n+1] != yr[_n]+1)
egen count1 = max(kill1), by(cnum)
recode count1 .=0
drop if yr <=count1

drop kill1 count1
sort cnum yr month


quie tab cnum
local noffirm = _result(2)

 /* some revland or refa only differ by 1 across _n and _n+1 */

quie by cnum: replace revland = revland[_n-1] if abs(revland[_n] - revland[_n-1]) <= 5
quie by cnum: replace refa = refa[_n-1] if abs(refa[_n] - refa[_n-1]) <=5

gen double refanol = refa - revland

 /* find out firms that trade lands frequently*/
quie by cnum: gen double chgland = (land[_n] - land[_n-1])/land[_n-1] if (land[_n] != 0)
recode chgland .=0
quie by cnum: gen tem1 = 1 if chgland > `upland' /* increase land by 200% */
recode tem1 . = 0
quie by cnum: gen tem2 = 1 if chgland < `dwnland' /* decrease land by 50% */
recode tem2 .=0
quie egen double tem11 = sum(tem1), by(cnum)
quie egen double tem21 = sum(tem2), by(cnum)
quie tab cnum if (tem11 >= 2)
disp _result(2)
quie tab cnum if (tem21 >= 2)
disp _result(2)
quie tab cnum if (tem11 >= 2) & (tem21 >= 2)
disp _result(2)


drop if (tem11 >=2) & (tem21 >=2)

drop tem1 tem2 tem11 tem21 chgland

sort cnum yr



 /* show # of firms never revaluate non-land asset */
quie egen double sum1 = sum(abs(refanol)), by(cnum)

quie tab cnum if sum1 == 0
disp "There are " _result(2) " out of " `noffirm' " firms that never did revaluation."

/* drop sum1 */
sort cnum yr month

quie gen REornot = 1 if sum1 == 1  /* a dummy used to distinguish whether a firm ever report revlauation on on-land asset */

/* drop if sum1 == 0 /* sum1 = 1: firm reporting revaluation */ */

drop sum1


fillin cnum yr  /* this expand obs of each firm to form a balanced set */
makep  /* fill in the price level for the just created obs */
sort cnum yr

/********** End of Data Filtering *********/



/****************** Block One *******************/
/** This block is to get the nonland no revaluation **/
/** asset, and use it to get the "old asset".  **/
/** The result is "oldgros".  *******************/
/************************************************/

quie gen double zbar = fa - land - revland + accdepre /* nonland gross asset */
quie gen double newbok = zbar - 5*refanol if /* refanol != 0  & */ refanol != . /* book value of new asset, not taken out depreciation */
quie gen tem1 = 1 if newbok < 0
quie recode tem1 . = 0
sort cnum yr

drop tem1 newbok zbar


/* create a variable of the pre-1981 vintage */

quie gen oldlife = `oldlive'  /* assume the old asset has remaing life of `oldlive' years. default = 6 */
quie gen wentby = yr - 1979  /* how many years went by the old asset since revaluated */
quie replace wentby = `oldlive'-1 if wentby >= `oldlive' /* imperfect, but otherwise oldnet would <0 for some */

sort cnum yr
quie by cnum: gen double Dreval = refanol[_n] - refanol[_n-1] if _n != 1  /* changes in the non-land revaluation */
quie by cnum: replace Dreval = refanol if (_n == 1) | ((_n != 1) & (refanol[_n] != .) & (refanol[_n-1] == .)) /* the first entry is "level", not "change" */
quie by cnum: gen tem1 = Dreval if Dreval >= 0 /* the part of increased revaluation */
quie recode tem1 . = 0

quie by cnum: replace tem1 = tem1*((2*`oldlive')/(2*`oldlive'-2*wentby-1))

quie by cnum: gen tem2 = Dreval if Dreval <= 0 /* the part of decreased revaluation */
quie recode tem2 . = 0

drop Dreval
sort cnum yr

quie egen double oldgros = sum(tem1), by(cnum) /* accumulation of the positive revaluation, multiplied by 4 */
                                          /* "4" because asset revaluate if price up to 1.25% */
                                          /* this measures the overall "old assets", before disposition and depreciation, in the 1979 dollar */
                                          /* this appends the same value to all periods */


quie replace oldgros = oldgros*4


sort cnum yr
quie by cnum: gen double oldgon1 = tem2*4 /* to be subtracted from DisFAnoL later */
quie by cnum: gen double oldgone = sum(tem2*4) /* negative; the accumulated book value (excluding reval) of old asset that is gone; in 1979 book value */
sort cnum yr

quie by cnum: replace oldgros = oldgros + oldgone /* the remains of pre-1979 asset, in 79 dollar */
                      /* net of dispositions but not net of depreciation; that is, it's book value */
                      /* ex, if 1990 has 145, means there is 145-Accumulated_Depreciaiton net */
                      /* values in 1979 dollar of the old asset before adjusting for inflation */
drop oldgone
sort cnum yr


/************* End of Block One **********************/
/*****************************************************/


/************* Block Two ****************************/
/** This block is to make adjustment of "oldgros", **/
/** and also to get "oldnet" which is net of       **/
/** depreciation and inflation adjusted.           **/
/** "oldgros" needs adjustment if the imputed      **/
/** oldgros is more than the total asset.          **/
/** The results are (adjusted) oldgros and oldnet. **/
/****************************************************/

/* begin error correction */

quie gen tem12 = yr if tem1 != 0
quie recode tem12 . = 99999
quie egen tem121 = min(tem12), by(cnum) /* the 1st time refanol appears */
sort cnum yr


quie by cnum: gen double tem122 =  oldgros /( fa - land - refa +accdepre) /* presumably < 1 */
quie by cnum: gen double tem123 = `mixrate'*(fa - land - refa + accdepre)/oldgros if tem122 > 1  /* the adjustment ratio */
quie recode tem123 .=1  /* if tem122 <=1, don't need adjustment, so the adjusment ratio is 1 */
quie egen tem124 = min(tem123), by(cnum) /* the bottom line of the adjustment ratio */
sort cnum yr
quie by cnum: replace oldgros = tem124*oldgros
sort cnum yr


drop tem12 tem121 tem122 tem123 tem124

drop tem1 tem2


/* compute the remaining values of old asset net of depreciation */

sort cnum yr
scalar avgP = (p1979 + p1978 + p1977 + p1976 + p1975)/5
quie gen double oldnet =(price/avgP)* oldgros*(2*oldlife-2*wentby-1)/(2*oldlife) /* the remaing net values of the old asset, in current year's dollar */
                 /* ex, if oldnet = 100 in 1990, it means the pre-1979 asset has a remaining */
                 /* net (of depreciation) value of 100 in 1990, in the 1990 year dollars */

quie gen double oldbok = oldnet* (avgP/price)



/*************** End of Block Two ******************/
/***************************************************/

drop oldlife wentby
sort cnum yr

/***************** Block Three ********************/
/** This block is to deal with the new asset     **/
/** by first figuring out the implied investment **/
/** series and also the dispositioned asset.     **/
/** The result is invest4 and DisFAnoD.          **/
/**************************************************/


 /* DisFA is the book value of disposed asset, including land. */

quie gen double DisFA = salefa + lstdisfa /* lstdisfa is "+" if loss money.  So Book - Sale = Lost. */

quie replace DisFA = 0 if (DisFA < 0 ) | (DisFA == .) /* replace the unreasonable number by 0 */

sort cnum yr
quie by cnum: gen double Dland = land[_n] - land[_n-1] /* change in land, not include revaluation */
quie gen Dland1 = Dland if Dland <=0 /* only register when land decreases */
quie recode Dland1 . = 0
quie gen Dland2 = Dland if Dland >=0 /* only register when land increases */
quie recode Dland2 . = 0
/* different; new. in that of refanol. */
quie by cnum: gen double invest2 = (fa[_n]-revland[_n]-refanol[_n]) - (fa[_n-1]-revland[_n-1]-refanol[_n-1]) + DisFA + deprec - Dland2
                         /* investment; by looking at changes in fa (which is net), excluding revaluations, adding back disposed asset and depreciations, and excluding lands */
quie by cnum: gen double invest3 = buyfa - Dland2
                         /* investment; by looking at the expenditure on purchasing fixed asset; excluding land */

quie gen invest4 = invest3
quie replace invest4 = invest2 if invest4 < 0
quie replace invest4 = buyfa if invest4 < 0

  /* the following expends the not-available investment data by assuming it is the */
  /* avarage of all available data */
quie egen double tem1 = mean(invest4) if _fillin == 0, by(cnum)
gsort cnum -yr
quie by cnum: replace tem1 = tem1[_n-1] if _fillin == 1 /* the "filled in" obs uses the same value as the one first available */
quie replace invest4 = tem1 if _fillin == 1
drop tem1 Dland Dland2 invest2 invest3

quie replace oldgros = 0 if REornot == 0

drop REornot

quie by cnum: gen double gk = fa - (land + revland) - refanol + accdepre - oldgros /* gross capital without land */
          /* or, the resulting is the book value of before-depreciation non-land NEW asset */
          /* it excludes the the book value of the gross pre-1979 asset, so it only has the gross post-1979 assets */
          /* "oldbok" is only for firms ever reporting revaluation */
quie gen invest = invest4 /* keep a virgin */
drop invest4 /* change the name to invest4 */
quie by cnum: gen double DisFAnoL = DisFA-(-Dland1) /* disposed asset, excluding land. note Dland1 < 0 */
quie replace DisFAnoL = 0 if DisFAnoL < 0  /* disposed asset net of land shouldn't be <0 */

quie gen DisFAnoD = DisFAnoL /* change the name from DisFAnoL to DisFAnoD because of the getnetk program */
drop DisFAnoL
quie gen newnetk = . /* create an empty list to be filled in in the following program */

drop oldgros
sort cnum yr

/******** End of Block Three ****************/
/********************************************/


/********************* Block Four *******************/
/** This block uses the program getnetk to get the **/
/** vintage of new assets and then computes the    **/
/** inflation and depreciation adjusted values of  **/
/** new capital.                                   **/
/** The result is newnetk.                         **/
/****************************************************/


global begyrg = `begyr'
global endyrg = `endyr'

capture program drop getnetk
program define getnetk
     local i $endyrg
     while `i' >=  $begyrg-2  {  /* "-2" because want to preserv more lags */

     gsort cnum -yr
     quie by cnum: gen gk`i' = gk[$endyrg - `i' +1 ] /* gross capial stock. implicitly, gk does not accounts for the discarded capital */

     gsort cnum -yr
     quie gen invest4 = invest if yr <= `i'  /* don't need 1997, 1996 data when deal with 1995 */
     quie by cnum: gen double invsum4 = sum(invest4) if invest4 != .

     gsort cnum -yr
     quie gen DisFAnoL = DisFAnoD if yr <= `i' /* duplicate DisFAnoL */
     quie by cnum: gen double discard  = sum(DisFAnoL) /* accumulated disposed asset excluding land */
                                           /* ex, if discard = 25 in 1992, meaning $25 assets are disposed btw 1992 and 1997 */
                                           /* however, we don't know whether the discarded is from old or new investment */

     quie by cnum: gen double gk`i'c = gk`i' + discard /* adding back the disposed asset, so gk97c accounts for depre and discard */

     gsort cnum -yr
     quie by cnum: gen date`i' = yr if (invsum4 >= gk`i'c) & (invsum4 != .)
     quie egen dat`i'm = max(date`i'), by(cnum) /* the year which accounts for 97 K stock */

     gsort cnum -yr
     quie by cnum: replace dat`i'm = 1981 if dat`i'm == . /* if beyond the oldest date available (1981), assume it is 1981 */

     quie gen invest41 = invest4 /* a dummy, used in fine-tunning the investment so that the nets are equal */

     gsort cnum -yr
     quie replace invest41 = gk`i'c[_n] - invsum4[_n-1] if yr == dat`i'm /* so that the sum of invest41 equals gk97c */
     quie gen invIN = invest41 if yr >= dat`i'm /* investment from the specific period that consists of the current period's stock */

     gsort cnum -yr

     /* it uses the same scheme no matter whether the date97m = "1981" is artifically imposed */
     quie by cnum: gen double invRem = invIN* (2*(`i' - dat`i'm +1) - 2*(`i' - yr) -1) / (2*(`i'-dat`i'm+1)) /* the remaing value of year-specific investment */
     quie egen double netk = sum(invRem) if invRem != ., by(cnum) /* imputed net book capital, doesn't matter yr or -yr */
                                             /* for each firm, netk has the same value in each period */

     sort cnum yr
     quie by cnum: gen double nk = fa - land - revland - refanol - oldbok /* after-depreciation before-inflation old and new assets, excluding land */
     quie replace nk = . if yr != `i' /* note implicitly nk takes into account the depreciation and salefa */


     gsort cnum -yr
     quie by cnum: gen double invRem1 = invRem* nk[$endyrg - `i' +1] / netk  /* the revised remaining values of in-place investment */

     scalar targetP = p`i'
     quie by cnum: gen double tem1 = invRem1*targetP/price /* inflated net new investment from each period */

     quie egen double tem2 = sum(tem1), by(cnum)

     gsort cnum -yr
     quie by cnum: replace newnetk = tem2 + oldnet if yr == `i'

     quie by cnum: replace newnetk = nk if (dat`i'm == `i') & (yr == `i') /* otherwise newnetk = 0 */
                                                            /* this happens when invest4 > nk (current investment is greater than the K stock) */

     drop gk`i' invest4 invsum4 DisFAnoL discard gk`i'c date`i' dat`i'm
     drop invest41 invIN invRem netk invRem1 tem1 tem2 nk
     scalar drop targetP

     local i = `i' - 1
}
end

getnetk

drop oldnet oldbok DisFAnoD
sort cnum yr

/************ Block Five ****************************************************/
/** This block is to deal with land.                                       **/
/** There are two parts.  The first is to get the revaluation number, not  **/
/** adjusting for changes in price level.  Variable landVal and the        **/
/** program adjRevl are for this purpose.  The second part is to adjust for **/
/** the price level.  Variable newVal, gamma, tau, phi, and the program    **/
/** landext is for this purpose.                                           **/
/****************************************************************************/


  /* first, round values of land and revland if the difference is <=5 */

gsort cnum -yr
quie by cnum: replace land = land[_n-1] if abs(land[_n] - land[_n-1])<=5
quie by cnum: replace revland = revland[_n-1] if abs(revland[_n] - revland[_n-1]) <=5

  /* create landVal: modify revland to take care of sequence-revaluation problem */

gsort cnum -yr
quie gen landVal = .
quie replace landVal = revland if yr == `endyr' /* the last year's (ex, 1996) value is set to the reported revland */

quie by cnum: gen tem1 = landVal if yr == `endyr'
quie recode tem1 . = 0
quie egen double tem2 = sum(tem1), by(cnum)  /* the value of landVal in the specified year */


sort cnum yr


capture program drop adjRevl  /* get landVal, which adjust for sequence-revaluation problem */
program define adjRevl
     local i  =$endyrg-1
     while `i' >= 1981  {

     capture drop tem3 tem4 tem5 tem6 tem7
     gsort cnum -yr
     quie by cnum: gen tem3 = 1 if yr > `i' & landVal == tem2 /* mark the years which have the same landVal as t-1 */
     quie recode tem3 . = 0
     sort cnum yr
     quie by cnum: gen tem4 = yr if tem3[_n]==0 & tem3[_n-1]==1 /* mark the year in which tem3 is discontinuous in 1; ex, 1 1 0 1 1: market the year of 0 */
     quie recode tem4 . = 99999
     quie egen tem5 = min(tem4), by(cnum)  /* the first break.  if tem3 is 0 0 1 1 1 (the last 1997), then tem4 = 99999 */
     sort cnum yr
     quie by cnum: replace tem3 = 0 if yr >= tem5 /* nullify tem3 after "break" */
     quie by cnum: gen tem6 = land if tem3 == 1 /* don't recode tem6; use . for very big number */
     quie egen tem7 = min(tem6), by(cnum) /* the minimum land in the interested periods */

     gsort cnum -yr
     quie by cnum: replace landVal = landVal[_n-1] if yr == `i' & revland[_n] < landVal[_n-1] & land[_n] >= tem7
     quie by cnum: replace landVal = revland if yr == `i' & landVal == .

     capture drop tem1 tem2
     quie by cnum: gen tem1 = landVal if yr == `i'
     quie recode tem1 . = 0
     quie egen double tem2 = sum(tem1), by(cnum)  /* the value of landVal in the specified year */

     local i = `i' -1
}
sort cnum yr
end

adjRevl


capture drop tem1 tem2


drop if land == .  /* drop those filled-in, assuming the 1st appearing year is the year of revaluation */
                    /* Otherwise, it probably amounts to saying no matter what, the revaluation year is in 1981 */

capture program drop pickyr  /* pick some important years for each firm, each obs.  */
program define pickyr
     {
     capture drop del1 del2 del3 pyr
     sort cnum yr

     quie by cnum: gen del1 = `3' if yr == `5'
     quie recode del1 . = 0
     quie egen double del2 = sum(del1), by(cnum) /* revland in the interested year */
     sort cnum yr
     quie by cnum: gen del3 = yr if `1' `2' del2 & yr `4' `5' & `1' != .
     quie recode del3 . = 0

     sort cnum yr
     quie by cnum: gen pyr = yr[1] if 1 `4' 2
     quie by cnum: replace pyr = yr[_N] if 2 `4' 1

     if 1 `4' 2{  /* for gamma and phi */
     capture drop del4 del5 del6 del7
     quie by cnum: gen del4 = yr if (del3[_n] == 0 | yr == yr[1]) & del3[_n+1]>1900 & del3[_n+1] != . /* mark the year in which del2 is discontinuous in 1 */
     quie recode del4 .=0
     quie egen del5 = max(del4), by(cnum) /* the year in which the first break occurs */
     sort cnum yr
     quie by cnum: replace del3 = 0 if yr < del5
     quie by cnum: replace del3 = 9999 if del3 == 0
     quie egen del6 = min(del3), by(cnum) /* the year of gamma */
     sort cnum yr
     quie by cnum: gen del7 =landP if yr == del6
     quie recode del7 . = 0
     quie egen double `6' = sum(del7), by(cnum)
     }

     if 2 `4' 1{   /* for tau */
     capture drop del4 del6 del7
     quie by cnum: gen del4 = yr if del3[_n] >1900 & del3[_n-1] == 0 & del3[_n] != . /* mark the year in which del2 is discontinuous in 1 */
     quie recode del4 .=0
     quie recode del4 0 = 9999
     quie egen del6 = min(del4), by(cnum)
     sort cnum yr
     quie by cnum: gen del7 = landP if yr == del6
     quie recode del7 . =0
     sort cnum yr
     quie egen double `6' = sum(del7), by(cnum)
     }

     sort cnum yr
     }
end


sort cnum yr
quie by cnum: gen newVal = .  /* create a dummy to be filled in */


capture program drop landext  /* the modified revaluation of the reported revaluation.   */
program define landext
     local i = 1982
     while `i' <= $endyrg{

     capture drop gamma tau phi
     sort cnum yr

     pickyr revland >= revland <= `i' gamma
     pickyr revland == landVal >= `i' tau
     pickyr landVal >  landVal <= `i' phi

     quie by cnum: replace newVal = landVal if yr == yr[1]  /* assuming the 1st revaluation year is the 1st year the firm appears */
     quie by cnum: replace newVal = revland*(landP/gamma) + (landVal - revland)*(landP/tau) if (yr == `i') & (revland < landVal) & (revland[_n] <= revland[_n-1]) & (revland[_n-1] != .)
     quie by cnum: replace newVal = revland[_n-1]*(landP/gamma) + (revland - revland[_n-1]) + (landVal - revland)*(landP/tau) if (yr == `i') & (revland < landVal) & (revland[_n] > revland[_n-1]) & (revland[_n-1] != .)
     quie by cnum: replace newVal = newVal[_n-1]*(landP/landP[_n-1]) if (yr == `i') & (revland == landVal) & (landVal == landVal[_n-1])
     quie by cnum: replace newVal = landVal*(landP/phi)              if (yr == `i') & (revland == landVal) & (landVal <  landVal[_n-1])
     quie by cnum: replace newVal = newVal[_n-1]*(landP/landP[_n-1]) + (landVal - landVal[_n-1]) if (yr == `i') & (revland == landVal) & (landVal > landVal[_n-1])

     local i = `i' + 1
}
end

landext

sort cnum yr
quie by cnum: gen landup = land if _n == 1


capture program drop updateL  /* adjusted for inflation */
program define updateL
     local i 1981 + 1
     while `i' <= $endyrg{
          sort cnum yr
          quie by cnum: replace landup = landup[_n-1]*(landP/landP[_n-1]) + (land - land[_n-1]) if _n > 1 & yr == `i' & land[_n] >= land[_n-1]
          quie by cnum: replace landup = landup[_n-1]*(landP/landP[_n-1])* (land/land[_n-1])    if _n > 1 & yr == `i' & land[_n] <  land[_n-1]
          local i = `i' + 1
     }
end

updateL


quie gen double newland = (landup + newVal)

 /* The following adjust inventory assuming it is */
 /* valued by the "average cost method"           */

sort cnum yr
quie by cnum: gen adjinv = inven if _n ==1

quie replace adjinv = inven*(2*price / (price + price[_n-1])) if _n >=2  /* Per Perfect and Wiles, 1994 */

quie gen double Wrpcst = newland + newnetk  /* the total replacement cost of capital */


/********************* End of Block Five ********************/
/************************************************************/

drop tem3 tem4 tem5 tem6 tem7 del1 del2 del3 del4 del5 del6 del7
drop gamma tau pyr phi

quie recode eqtymkt2 0=. /* 0: not publicly traded */

scalar lg1 = 1  /* Fazzari et al. [1988] use beginning of the period K */
scalar lg2 = 0  /* whether 1 or 0 doesn't change results much */

sort cnum yr
quie by cnum: gen double Wq   = (eqtymkt2[_n-lg2]*1000 + stdebt + ltdebt)/(Wrpcst[_n-lg1]+adjinv[_n-lg1])

sort cnum yr month
